<?php
/**
 * CustomerData File Doc Comment
 * PHP Version 7.1
 *
 * @category CustomerData
 * @package  EasySMS
 * @author   Tern <info@tern.gr>
 * @license  https://easysms.gr/license EasySms
 * @link     https://easysms.gr
 */
namespace Tern\Smsplugin\Helper;

use Magento\Framework\Event\Observer;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Sales\Model\Order;
use Magento\Framework\App\Helper\Context;
use Magento\Customer\Api\CustomerRepositoryInterface as CustomerInterface;
use Magento\Customer\Api\AddressRepositoryInterface as AddressInterface;

/**
 * CustomerData class Doc Comment
 * PHP Version 7.1
 *
 * @category CustomerData
 * @package  EasySMS
 * @author   Tern <info@tern.gr>
 * @license  https://easysms.gr/license EasySms
 * @link     https://easysms.gr
 */
class CustomerData extends AbstractHelper
{
    protected $order;
    protected $customerInterface;
    protected $addressInterface;

    protected $easysms_apibasepath = 'https://www.easysms.gr/api/';
    protected $easysms_api_sendsms = 'sms/send';
    protected $easysms_api_callback 
        = 'http://163.172.111.60/magento/magento2/tern_smsplugin';

    /**
     * Constructor
     *
     * @param Context           $context           context
     * @param Order             $order             order
     * @param CustomerInterface $customerInterface customerInterface
     * @param AddressInterface  $addressInterface  addressInterface
     */
    public function __construct(
        Context $context,
        Order $order,
        CustomerInterface $customerInterface,
        AddressInterface $addressInterface
    ) {
        $this->order = $order;
        $this->customerInterface = $customerInterface;
        $this->addressInterface = $addressInterface;
        parent::__construct($context);
    }


    /**
     * Getting Order Details
     *
     * @param Observer $observer event observer
     *
     * @return string
     */
    public function getOrder(Observer $observer)
    {
        $order              = $observer->getEvent()->getOrder();
        $orderId            = $order->getIncrementId();
        $orderInformation   = $this->order->loadByIncrementId($orderId);
        return $orderInformation;
    }

    /**
     * Getting Order Data
     *
     * @param \Magento\Sales\Model\Order $order order
     *
     * @return array
     */
    public function getOrderData(\Magento\Sales\Model\Order $order)
    {
        $orderData = [
            'order_id' => $order->getIncrementId(),
            'firstname' => $order->getCustomerFirstname(),
            'middlename' => $order->getCustomerMiddlename(),
            'lastname' => $order->getCustomerLastname(),
            'totalPrice' => number_format($order->getGrandTotal(), 2),
            'currency' => $order->getOrderCurrencyCode(),
            'email' => $order->getCustomerEmail(),
            'status' => $order->getState()
        ];

        return $orderData;
    }

    /**
     * Getting Order Data
     *
     * @param Observer $observer event observer
     *
     * @return array
     */
    public function getCustomerData(Observer $observer)
    {
        $customer = $observer->getEvent()->getCustomer();

        $customerData = [
            'order_id' => "",
            'firstname' => $customer->getFirstname(),
            'middlename' => "",
            'lastname' => $customer->getLastname(),
            'totalPrice' => "",
            'currency' => "",
            'email' => $customer->getEmail(),
            'status' => ""
        ];

        return $customerData;
    }

    /**
     * Getting phone number of customer
     *
     * @param Observer $observer observer
     *
     * @return string
     */
    public function getCustomerPhone(Observer $observer)
    {
        $order = $this->getOrder($observer);
        $phone = null;
        $phone = $order->getCustomerPhone();
        if ($phone === null ) {
            $billingAddress = $order->getBillingAddress();
            if ($billingAddress !== null) {
                $phone = $billingAddress->getTelephone();
            }
        }
        if ($phone === null) {
            $billingAddress = $order->getShippingAddress();
            if ($billingAddress !== null) {
                $phone = $billingAddress->getTelephone();
            }
        }
        if ($phone === null) {
            $billingAddress = $order->getPrimaryBillingAddress();
            if ($billingAddress !== null) {
                $phone = $billingAddress->getTelephone();
            }
        }

        if ($phone === null) {
            $data = $observer->getEvent()->getOrder();
            foreach ($data['addresses'] as $addr) {
                if ($addr['telephone'] !== null ) {
                    $phone = $addr['telephone'];
                    break;
                }
            }
        }

        return $phone;
    }

    /**
     * Checks the path is of a new order
     *
     * @param @var $path path
     *
     * @return boolean
     */
    public function isNewOrder($path)
    {
        // front end path: 
        // /magento/magento2/rest/default/V1/carts/mine/payment-information
        if (strpos($path, "/rest/")) {
            return true;
            // admin path: /magento/magento2/admin_awx71o/sales/order_create/save/
        } elseif (strpos($path, "/sales/order_create/")) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Checks the path is of an order status change
     *
     * @param @var $path path
     *
     * @return boolean
     */
    public function isOrderStatus($path)
    {
        // mass hold: /magento/magento2/admin_awx71o/sales/order/massHold/key/
        if (strpos($path, "/order/massHold/")) {
            return true;

            // hold: /magento/magento2/admin_awx71o/sales/order/hold/
        } elseif (strpos($path, "/order/hold/")) {
            return true;

            // mass unhold: /magento/magento2/admin_awx71o/sales/order/massUnhold/
        } elseif (strpos($path, "/order/massUnhold/")) {
            return true;

            // unhold: /magento/magento2/admin_awx71o/sales/order/hold/
        } elseif (strpos($path, "/order/unhold/")) {
            return true;

            // mass cancel: /magento/magento2/admin_awx71o/sales/order/massCancel/
        } elseif (strpos($path, "/order/massCancel/")) {
            return true;

            // cancel: /magento/magento2/admin_awx71o/sales/order/cancel/
        } elseif (strpos($path, "/order/cancel/")) {
            return true;

        } else {
            return false;
        }
    }


    /**
     * Insert Admin Config Values in the message using order data
     *
     * @param @var $message message
     * @param @var $data    data
     *
     * @return string
     */
    public function replaceKeywords($message, $data)
    {
        $keywords   = [
            '{order_id}',
            '{firstname}',
            '{middlename}',
            '{lastname}',
            '{totalPrice}',
            '{currency}',
            '{email}',
            '{status}'
        ];
        $message = str_replace($keywords, $data, $message);
        return $message;
    }

    /**
     * Retrieve the customer's phone number (if any) from the customer's billing
     * or shipping address
     *
     * @param @var $customer_id customer_id
     *
     * @return string
     */
    public function getPhoneFromAddress($customer_id)
    {
        $customer = $this->customerInterface->getById($customer_id);
        $billingAddressId = $customer->getDefaultBilling();
        $shippingAddressId = $customer->getDefaultShipping();

        try {
            $billingAddress = $this->addressInterface
                ->getById($billingAddressId);
            return $billingAddress->getTelephone();
        } catch (\Exception $e) {
            try{
                $billingAddress = $this->addressInterface
                    ->getById($shippingAddressId);
                return $billingAddress->getTelephone();
            } catch (\Exception $e2) {
                return null;
            }
        }
    }
}



