<?php
/**
 * Easysms Repository
 * PHP Version 7.1
 *
 * @category EasysmsRepository
 * @package  EasySms
 * @author   Tern <info@tern.gr>
 * @license  https://easysms.gr/license EasySms
 * @link     https://easysms.gr
 */
namespace Tern\Smsplugin\Model;

use Tern\Smsplugin\Api\EasysmsRepositoryInterface;
use Tern\Smsplugin\Api\Data\EasysmsInterface;
use Tern\Smsplugin\Model\EasysmsFactory;
use Tern\Smsplugin\Model\ResourceModel\Easysms\CollectionFactory;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Api\SearchResultsInterfaceFactory;

/**
 * EasysmsRepository Class Doc Comment
 *
 * @category Class
 * @package  EasySms
 * @author   Tern <info@tern.gr>
 * @license  https://easysms.gr/license EasySms
 * @link     https://easysms.gr
 */
class EasysmsRepository implements \Tern\Smsplugin\Api\EasysmsRepositoryInterface
{
    protected $objectFactory;
    protected $collectionFactory;

    /**
    * Function __construct
    *
    * Constructor
    *
    * @param EasysmsFactory                $objectFactory        objectFactory
    * @param CollectionFactory             $collectionFactory    collectionFactory
    * @param SearchResultsInterfaceFactory $searchResultsFactory factory     
    *
    * @return object
    */
    public function __construct(
        EasysmsFactory $objectFactory,
        CollectionFactory $collectionFactory,
        SearchResultsInterfaceFactory $searchResultsFactory       
    ) {
    
        $this->objectFactory        = $objectFactory;
        $this->collectionFactory    = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
    }
    
    /**
    * Function save
    *
    * Saves the object
    *
    * @param EasysmsInterface $object object
    *
    * @return object
    */
    public function save(EasysmsInterface $object)
    {
        try
        {
            $object->save();
        }
        catch(\Exception $e)
        {
            throw new CouldNotSaveException(__($e->getMessage()));
        }
        return $object;
    }

    /**
    * Function getById
    *
    * @param @var $id id
    *
    * @return EasySms
    */
    public function getById($id)
    {
        $object = $this->objectFactory->create();
        $object->load($id);
        if (!$object->getId()) {
            throw new NoSuchEntityException(
                __('Object with id "%1" does not exist.', $id)
            );
        }
        return $object;        
    }       

    /**
    * Function delete
    *
    * @param @var $object object
    *
    * @return EasySms
    */
    public function delete(EasysmsInterface $object)
    {
        try {
            $object->delete();
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__($exception->getMessage()));
        }
        return true;    
    }    

    /**
    * Function deleteById
    *
    * @param @var $id id
    *
    * @return EasySms
    */
    public function deleteById($id)
    {
        return $this->delete($this->getById($id));
    }    

    /**
    * Function getList
    *
    * @param @var $criteria criteria
    *
    * @return EasySms
    */
    public function getList(SearchCriteriaInterface $criteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);  
        $collection = $this->collectionFactory->create();
        foreach ($criteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType()
                    ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];
            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }  
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $criteria->getSortOrders();
        if ($sortOrders) {
            foreach ($sortOrders as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection()
                        == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($criteria->getCurrentPage());
        $collection->setPageSize($criteria->getPageSize());
        $objects = [];                                     
        foreach ($collection as $objectModel) {
            $objects[] = $objectModel;
        }
        $searchResults->setItems($objects);
        return $searchResults;        
    }
}
